<!doctype html>
<head>
  <title>kobo downloader</title>
  <link href="https://unpkg.com/tailwindcss@^1.0/dist/tailwind.min.css" rel="stylesheet">
</head>
<body>
  <nav class="flex items-center justify-between flex-wrap bg-red-500 py-2 px-6">
    <div class="flex items-center flex-shrink-0 text-white mr-6">
      <span class="font-semibold text-xl tracking-tight">Kobo Downloader</span>
    </div>
    <div class="flex-grow flex items-center w-auto">
      <div class="text-base flex-grow">
        <a href="/user" class="mt-0 inline-block text-red-200 hover:text-white mr-4">
          Users
        </a>
        <a href="/book" class="inline-block mt-0 text-red-200 hover:text-white mr-4">
          All Books
        </a>
      </div>
      <div>
        <a
          href="https://github.com/subdavis/kobo-book-downloader"
          class="inline-block text-sm px-4 py-2 leading-none border rounded text-white border-white
            hover:border-transparent hover:text-red-500 hover:bg-white mt-0"
        >
          GitHub
        </a>
      </div>
    </div>
  </nav>
  <div class="container mx-auto">
